@@ -670,6 +670,16 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                                 and (orca_state.activeScript.app \
                                      != event.host_application)
 
+                    # One last check -- let's make sure the new script actually
+                    # has an active window.  See bug #568696.
+                    #
+                    if setNewActiveScript and event.host_application:
+                        setNewActiveScript = False
+                        for child in event.host_application:
+                            if child.getState().contains(pyatspi.STATE_ACTIVE):
+                                setNewActiveScript = True
+                                break
+
                     if not reason and setNewActiveScript:
                         reason = "object received focus"
 
@@ -751,7 +761,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             event = e
         elif isinstance(e, input_event.BrailleEvent):
             debug.println(debug.LEVEL_ALL,
-                          "----------> QUEUEING BRAILLE COMMAND %d" % e.event)
+                          "----------> QUEUEING BRAILLE COMMAND %s" \
+                          % repr(e.event))
             event = e
         else:
             if e.type in settings.ignoredEventsList:
@@ -894,15 +905,15 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                               % (pressRelease, event.event_string))
             elif isinstance(event, input_event.BrailleEvent):
                 debug.println(debug.LEVEL_ALL,
-                              "DEQUEUED BRAILLE COMMAND %d <----------" \
-                              % event.event)
+                              "DEQUEUED BRAILLE COMMAND %s <----------" \
+                              % repr(event.event))
                 debug.println(debug.eventDebugLevel,
-                              "\nvvvvv PROCESS BRAILLE EVENT %d vvvvv"\
-                              % event.event)
+                              "\nvvvvv PROCESS BRAILLE EVENT %s vvvvv"\
+                              % repr(event.event))
                 self._processBrailleEvent(event)
                 debug.println(debug.eventDebugLevel,
-                              "\n^^^^^ PROCESS BRAILLE EVENT %d ^^^^^"\
-                              % event.event)
+                              "\n^^^^^ PROCESS BRAILLE EVENT %s ^^^^^"\
+                              % repr(event.event))
             else:
                 if (not debug.eventDebugFilter) \
                     or (debug.eventDebugFilter \
