@@ -19,9 +19,9 @@
 
 """Custom script for Evolution."""
 
-__id__        = "$Id: script.py 4440 2009-01-20 22:06:12Z joanied $"
-__version__   = "$Revision: 4440 $"
-__date__      = "$Date: 2009-01-20 17:06:12 -0500 (Tue, 20 Jan 2009) $"
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -41,8 +41,8 @@ import orca.settings as settings
 
 from orca.orca_i18n import _ # for gettext support
 
-from where_am_i import WhereAmI
 from speech_generator import SpeechGenerator
+from formatting import Formatting
 ########################################################################
 #                                                                      #
 # The Evolution script class.                                          #
@@ -69,6 +69,10 @@ class Script(default.Script):
 
         self.message_panel = None
 
+        # A handle to the Spellcheck dialog.
+        #
+        self.spellCheckDialog = None
+
         # Dictionary of Setup Assistant panels already handled.
         #
         self.setupPanels = {}
@@ -135,11 +139,9 @@ class Script(default.Script):
 
         return SpeechGenerator(self)
 
-    def getWhereAmI(self):
-        """Returns the "where am I" class for this script.
-        """
-
-        return WhereAmI(self)
+    def getFormatting(self):
+        """Returns the formatting strings for this script."""
+        return Formatting(self)
 
     def setupInputEventHandlers(self):
         """Defines InputEventHandler fields for this script that can be
@@ -279,9 +281,9 @@ class Script(default.Script):
                                                    getState().contains( \
                                        pyatspi.STATE_SENSITIVE)):
                                     self.updateBraille(orca_state.locusOfFocus)
-                                    speech.speakUtterances(
-                                        self.speechGenerator.getSpeech( \
-                                            orca_state.locusOfFocus, False))
+                                    speech.speak(
+                                        self.speechGenerator.generateSpeech(
+                                            orca_state.locusOfFocus))
             except NotImplementedError:
                 pass
 
@@ -333,11 +335,11 @@ class Script(default.Script):
 
         savedSpeechVerbosityLevel = settings.speechVerbosityLevel
         settings.speechVerbosityLevel = settings.VERBOSITY_LEVEL_BRIEF
-        utterances = speechGen.getSpeech(tab, False)
-        speech.speakUtterances(utterances)
+        utterances = speechGen.generateSpeech(tab)
+        speech.speak(utterances)
         settings.speechVerbosityLevel = savedSpeechVerbosityLevel
 
-        braille.displayRegions(brailleGen.getBrailleRegions(tab))
+        braille.displayRegions(brailleGen.generateBraille(tab))
 
     def getTimeForCalRow(self, row, noIncs):
         """Return a string equivalent to the time of the given row in
@@ -366,6 +368,149 @@ class Script(default.Script):
 
         return hrs + ' ' + mins + ' ' + suffix
 
+    def getAllSelectedText(self, obj):
+        """Get all the text applicable text selections for the given object.
+        If there is selected text, look to see if there are any previous
+        or next text objects that also have selected text and add in their
+        text contents.
+
+        Arguments:
+        - obj: the text object to start extracting the selected text from.
+
+        Returns: all the selected text contents plus the start and end
+        offsets within the text for the given object.
+        """
+
+        textContents = ""
+        startOffset = 0
+        endOffset = 0
+        if obj.queryText().getNSelections() > 0:
+            [textContents, startOffset, endOffset] = \
+                                            self.getSelectedText(obj)
+
+        # Unfortunately, Evolution doesn't use the FLOWS_FROM and
+        # FLOWS_TO relationships to easily allow us to get to previous
+        # and next text objects. Instead we have to move up the
+        # component hierarchy until we get to the object containing all
+        # the panels (with each line containing a single text item).
+        # We can then check in both directions to see if there is other
+        # contiguous text that is selected. We also have to jump over
+        # zero length (empty) text lines and continue checking on the
+        # other side.
+        #
+        container = obj.parent.parent
+        current = obj.parent.getIndexInParent()
+        morePossibleSelections = True
+        while morePossibleSelections:
+            morePossibleSelections = False
+            if (current-1) >= 0:
+                prevPanel = container[current-1]
+                try:
+                    prevObj = prevPanel[0]
+                    displayedText = prevObj.queryText().getText(0, -1)
+                    if len(displayedText) == 0:
+                        current -= 1
+                        morePossibleSelections = True
+                    elif prevObj.queryText().getNSelections() > 0:
+                        [newTextContents, start, end] = \
+                                     self.getSelectedText(prevObj)
+                        textContents = newTextContents + " " + textContents
+                        current -= 1
+                        morePossibleSelections = True
+                except:
+                    pass
+
+        current = obj.parent.getIndexInParent()
+        morePossibleSelections = True
+        while morePossibleSelections:
+            morePossibleSelections = False
+            if (current+1) < container.childCount:
+                nextPanel = container[current+1]
+                try:
+                    nextObj = nextPanel[0]
+                    displayedText = nextObj.queryText().getText(0, -1)
+                    if len(displayedText) == 0:
+                        current += 1
+                        morePossibleSelections = True
+                    elif nextObj.queryText().getNSelections() > 0:
+                        [newTextContents, start, end] = \
+                                     self.getSelectedText(nextObj)
+                        textContents += " " + newTextContents
+                        current += 1
+                        morePossibleSelections = True
+                except:
+                    pass
+
+        return [textContents, startOffset, endOffset]
+
+    def hasTextSelections(self, obj):
+        """Return an indication of whether this object has selected text.
+        Note that it's possible that this object has no text, but is part
+        of a selected text area. Because of this, we need to check the
+        objects on either side to see if they are none zero length and
+        have text selections.
+
+        Arguments:
+        - obj: the text object to start checking for selected text.
+
+        Returns: an indication of whether this object has selected text,
+        or adjacent text objects have selected text.
+        """
+
+        currentSelected = False
+        otherSelected = False
+        nSelections = obj.queryText().getNSelections()
+        if nSelections:
+            currentSelected = True
+        else:
+            otherSelected = False
+            displayedText = obj.queryText().getText(0, -1)
+            if len(displayedText) == 0:
+                container = obj.parent.parent
+                current = obj.parent.getIndexInParent()
+                morePossibleSelections = True
+                while morePossibleSelections:
+                    morePossibleSelections = False
+                    if (current-1) >= 0:
+                        prevPanel = container[current-1]
+                        prevObj = prevPanel[0]
+                        try:
+                            prevObjText = prevObj.queryText()
+                        except:
+                            prevObjText = None
+
+                        if prevObj and prevObjText:
+                            if prevObjText.getNSelections() > 0:
+                                otherSelected = True
+                            else:
+                                displayedText = prevObjText.getText(0, -1)
+                                if len(displayedText) == 0:
+                                    current -= 1
+                                    morePossibleSelections = True
+
+                current = obj.parent.getIndexInParent()
+                morePossibleSelections = True
+                while morePossibleSelections:
+                    morePossibleSelections = False
+                    if (current+1) < container.childCount:
+                        nextPanel = container[current+1]
+                        nextObj = nextPanel[0]
+                        try:
+                            nextObjText = nextObj.queryText()
+                        except:
+                            nextObjText = None
+
+                        if nextObj and nextObjText:
+                            if nextObjText.getNSelections() > 0:
+                                otherSelected = True
+                            else:
+                                displayedText = nextObjText.getText(0, -1)
+                                if len(displayedText) == 0:
+                                    current += 1
+                                    morePossibleSelections = True
+
+        return [currentSelected, otherSelected]
+
     def textLines(self, obj):
         """Creates a generator that can be used to iterate over each line
         of a text object, starting at the caret offset.
@@ -492,7 +637,7 @@ class Script(default.Script):
             #print "COMPLETED", context.utterance, context.currentOffset
             obj = context.obj[len(context.obj)-1]
             obj.queryText().setCaretOffset(context.currentOffset)
-            orca.setLocusOfFocus(None, obj, False)
+            orca.setLocusOfFocus(None, obj, notifyPresentationManager=False)
 
         # If there is a selection, clear it. See bug #489504 for more details.
         # This is not straight forward with Evolution. all the text is in
@@ -513,6 +658,81 @@ class Script(default.Script):
                     if text.getNSelections():
                         text.removeSelection(0)
 
+    def isSpellingSuggestionsList(self, obj):
+        """Returns True if obj is the list of spelling suggestions
+        in the spellcheck dialog.
+
+        Arguments:
+        - obj: the Accessible object of interest.
+        """
+
+        # The list of spelling suggestions is a table whose parent is
+        # a scroll pane. This in and of itself is not sufficiently
+        # unique. What makes the spell check dialog unique is the
+        # quantity of push buttons found. If we find this combination,
+        # we'll assume its the spelling dialog.
+        #
+        if obj and obj.getRole() == pyatspi.ROLE_TABLE_CELL:
+            obj = obj.parent
+
+        if not obj \
+           or obj.getRole() != pyatspi.ROLE_TABLE \
+           or obj.parent.getRole() != pyatspi.ROLE_SCROLL_PANE:
+            return False
+
+        topLevel = self.getTopLevel(obj.parent)
+        if not self.isSameObject(topLevel, self.spellCheckDialog):
+            # The group of buttons is found in a filler which is a
+            # sibling of the scroll pane.
+            #
+            for sibling in obj.parent.parent:
+                if sibling.getRole() == pyatspi.ROLE_FILLER:
+                    buttonCount = 0
+                    for child in sibling:
+                        if child.getRole() == pyatspi.ROLE_PUSH_BUTTON:
+                            buttonCount += 1
+                    if buttonCount >= 5:
+                        self.spellCheckDialog = topLevel
+                        return True
+        else:
+            return True
+
+        return False
+
+    def getMisspelledWordAndBody(self, suggestionsList, messagePanel):
+        """Gets the misspelled word from the spelling dialog and the
+        list of words from the message body.
+
+        Arguments:
+        - suggestionsList: the list of spelling suggestions from the
+          spellcheck dialog
+        - messagePanel: the panel containing the message being checked
+          for spelling
+
+        Returns [mispelledWord, messageBody]
+        """
+
+        misspelledWord, messageBody = "", []
+
+        # Look for the "Suggestions for "xxxxx" label in the spell
+        # checker dialog panel. Extract out the xxxxx. This will be
+        # the misspelled word.
+        #
+        text = self.getDisplayedLabel(suggestionsList) or ""
+        words = text.split()
+        for word in words:
+            if word[0] in ["'", '"']:
+                misspelledWord = word[1:len(word) - 1]
+                break
+
+        if messagePanel != None:
+            allTextObjects = self.findByRole(messagePanel, pyatspi.ROLE_TEXT)
+            for obj in allTextObjects:
+                for word in self.getText(obj, 0, -1).split():
+                    messageBody.append(word)
+
+        return [misspelledWord, messageBody]
+
     def isMessageBodyText(self, obj):
         """Returns True if obj is in the body of an email message.
 
@@ -692,7 +912,7 @@ class Script(default.Script):
                         utterances.append(string)
 
                 braille.displayRegions([regions, regions[0]])
-                speech.speakUtterances(utterances)
+                speech.speak(utterances)
                 return
 
         # 3) Mail view: message header list
@@ -863,16 +1083,19 @@ class Script(default.Script):
                             settings.speechVerbosityLevel = \
                                 settings.VERBOSITY_LEVEL_BRIEF
 
-                            utterances = speechGen.getSpeech(header, False)
+                            utterances = speechGen.generateSpeech(
+                                header,
+                                includeContext=False,
+                                priorObj=oldLocusOfFocus)
                             [headerRegions, focusedRegion] = \
-                                         brailleGen.getBrailleRegions(header)
+                                         brailleGen.generateBraille(header)
                             brailleRegions.extend(headerRegions)
                             brailleRegions.append(braille.Region(" "))
 
                             if column == i:
                                 cellWithFocus = focusedRegion
                             if speakAll or (column == i):
-                                speech.speakUtterances(utterances)
+                                speech.speak(utterances)
 
                         # Speak/braille the table cell.
                         #
@@ -889,9 +1112,12 @@ class Script(default.Script):
                                 settings.VERBOSITY_LEVEL_BRIEF
                         settings.speechVerbosityLevel = \
                             savedSpeechVerbosityLevel
-                        utterances = speechGen.getSpeech(cell, False)
+                        utterances = speechGen.generateSpeech(
+                            cell,
+                            includeContext=False,
+                            priorObj=oldLocusOfFocus)
                         [cellRegions, focusedRegion] = \
-                                           brailleGen.getBrailleRegions(cell)
+                                           brailleGen.generateBraille(cell)
 
                         # Translators: this is the name of the
                         # status column header in the message
@@ -941,7 +1167,7 @@ class Script(default.Script):
                             cellWithFocus = focusedRegion
 
                         if speakAll or (column == i):
-                            speech.speakUtterances(utterances)
+                            speech.speak(utterances)
 
             if brailleRegions != []:
                 braille.displayRegions([brailleRegions, cellWithFocus])
@@ -980,10 +1206,12 @@ class Script(default.Script):
                           + "day view: tabbing to day with appts.")
 
             parent = event.source.parent
-            utterances = speechGen.getSpeech(parent, False)
+            utterances = speechGen.generateSpeech(parent,
+                                                  includeContext=False,
+                                                  priorObj=oldLocusOfFocus)
             [brailleRegions, focusedRegion] = \
-                    brailleGen.getBrailleRegions(parent)
-            speech.speakUtterances(utterances)
+                    brailleGen.generateBraille(parent)
+            speech.speak(utterances)
 
             apptExtents = event.source.queryComponent().getExtents(0)
 
@@ -996,12 +1224,14 @@ class Script(default.Script):
                         appt = childTable.getAccessibleAt(row, 0)
                         extents = appt.queryComponent().getExtents(0)
                         if extents.y == apptExtents.y:
-                            utterances = speechGen.getSpeech(event.source, \
-                                                             False)
+                            utterances = speechGen.generateSpeech(
+                                event.source,
+                                includeContext=False,
+                                priorObj=oldLocusOfFocus)
                             [apptRegions, focusedRegion] = \
-                                brailleGen.getBrailleRegions(event.source)
+                                brailleGen.generateBraille(event.source)
                             brailleRegions.extend(apptRegions)
-                            speech.speakUtterances(utterances)
+                            speech.speak(utterances)
 
                             startTime = 'Start time ' + \
                                 self.getTimeForCalRow(j, noRows)
@@ -1058,11 +1288,14 @@ class Script(default.Script):
                     apptExtents = child.queryComponent().getExtents(0)
 
                     if extents.y == apptExtents.y:
-                        utterances = speechGen.getSpeech(child, False)
+                        utterances = speechGen.generateSpeech(
+                            child,
+                            includeContext=False,
+                            priorObj=oldLocusOfFocus)
                         [apptRegions, focusedRegion] = \
-                            brailleGen.getBrailleRegions(child)
+                            brailleGen.generateBraille(child)
                         brailleRegions.extend(apptRegions)
-                        speech.speakUtterances(utterances)
+                        speech.speak(utterances)
 
                         startTime = 'Start time ' + \
                             self.getTimeForCalRow(index, noRows)
@@ -1203,7 +1436,7 @@ class Script(default.Script):
                                            "Home", "End", "Return", "Tab"]:
                         return
 
-                    # If the last keyboard event was a "same line" 
+                    # If the last keyboard event was a "same line"
                     # navigation key, then pass this event onto the
                     # onCaretMoved() method in the parent class for
                     # speaking. See bug #516565 for more details.
@@ -1226,15 +1459,8 @@ class Script(default.Script):
         # caret currently is, and use this to speak a selection of the
         # surrounding text, to give the user context for the current misspelt
         # word.
-
-        self.rolesList = [pyatspi.ROLE_TABLE, \
-                         pyatspi.ROLE_SCROLL_PANE, \
-                         pyatspi.ROLE_PANEL, \
-                         pyatspi.ROLE_PANEL, \
-                         pyatspi.ROLE_PANEL, \
-                         pyatspi.ROLE_FILLER, \
-                         pyatspi.ROLE_DIALOG]
-        if self.isDesiredFocusedItem(event.source, self.rolesList):
+        if self.isSpellingSuggestionsList(event.source) \
+           or self.isSpellingSuggestionsList(newLocusOfFocus):
             debug.println(self.debugLevel,
                       "evolution.locusOfFocusChanged - spell checking dialog.")
 
@@ -1242,42 +1468,11 @@ class Script(default.Script):
             #
             self.updateBraille(orca_state.locusOfFocus)
 
-            # Look for the "Suggestions for 'xxxxx' label in the spell
-            # checker dialog panel. Extract out the xxxxx. This will be the
-            # misspelt word.
-            #
-            panel = event.source.parent.parent
-            allLabels = self.findByRole(panel, pyatspi.ROLE_LABEL)
-            found = False
-            for label in allLabels:
-                if not found:
-                    text = self.getDisplayedText(label)
-                    if text:
-                        tokens = text.split()
-                    else:
-                        tokens = []
-                    for token in tokens:
-                        if token.startswith("'"):
-                            badWord = token
-                            badWord = badWord[1:len(badWord)-1]
-                            found = True
-                            break
-
-            # If we have a handle to the HTML message panel, then extract out
-            # all the text objects, and create a list of all the words found
-            # in them.
-            #
-            if self.message_panel != None:
-                allTokens = []
-                panel = self.message_panel
-                allText = self.findByRole(panel, pyatspi.ROLE_TEXT)
-                for i in range(0, len(allText)):
-                    text = self.getText(allText[i], 0, -1)
-                    tokens = text.split()
-                    allTokens += tokens
-
+            if not self.pointOfReference.get('activeDescendantInfo'):
+                [badWord, allTokens] = \
+                    self.getMisspelledWordAndBody(event.source,
+                                                  self.message_panel)
                 self.speakMisspeltWord(allTokens, badWord)
-                return
 
         # 10) Mail view: message area - attachments.
         #
@@ -1448,8 +1643,8 @@ class Script(default.Script):
         # prior to speaking the actual message that gets focus.
         # See bug #347964.
         #
-        if isinstance(orca_state.lastInputEvent,
-                      input_event.KeyboardEvent):
+        if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent) \
+           and orca_state.lastNonModifierKeyEvent:
             string = orca_state.lastNonModifierKeyEvent.event_string
             if string == "Delete":
                 rolesList = [pyatspi.ROLE_TABLE_CELL, \
@@ -1474,6 +1669,74 @@ class Script(default.Script):
         #
         default.Script.onFocus(self, event)
 
+    def onActiveDescendantChanged(self, event):
+        """Called when an object who manages its own descendants detects a
+        change in one of its children.
+
+        Arguments:
+        - event: the Event
+        """
+
+        # The default script's onActiveDescendantChanged method is cutting
+        # off speech with a speech.stop. If we're in the spellcheck dialog,
+        # this interrupts the presentation of the context.
+        #
+        if self.isSpellingSuggestionsList(event.source):
+            orca.setLocusOfFocus(event, event.any_data)
+
+            # We'll tuck away the activeDescendant information for future
+            # reference since the AT-SPI gives us little help in finding
+            # this.
+            #
+            if orca_state.locusOfFocus \
+               and (orca_state.locusOfFocus != event.source):
+                self.pointOfReference['activeDescendantInfo'] = \
+                    [orca_state.locusOfFocus.parent,
+                     orca_state.locusOfFocus.getIndexInParent()]
+            return
+
+        default.Script.onActiveDescendantChanged(self, event)
+
+    def onTextInserted(self, event):
+        """Called whenever text is inserted into an object.
+
+        Arguments:
+        - event: the Event
+        """
+
+        # When the active descendant in the list of misspelled words
+        # changes, we typically get an object:active-descendant-changed
+        # event. Unfortunately, we don't seem to get this event (or a
+        # focus: event) when the user presses a button without moving
+        # focus there explicitly. (e.g. pressing Alt+R) The label which
+        # is associated with the spelling list gets new text. So we'll
+        # try to look for that instead.
+        #
+        if event.source.getRole() == pyatspi.ROLE_LABEL:
+            relations = event.source.getRelationSet()
+            for relation in relations:
+                if relation.getRelationType() == pyatspi.RELATION_LABEL_FOR:
+                    target = relation.getTarget(0)
+                    if self.isSpellingSuggestionsList(target):
+                        [badWord, allTokens] = \
+                            self.getMisspelledWordAndBody(target,
+                                                          self.message_panel)
+                        self.speakMisspeltWord(allTokens, badWord)
+
+                        try:
+                            selection = target.querySelection()
+                        except NotImplementedError:
+                            selection = None
+                        if selection and selection.nSelectedChildren > 0:
+                            newFocus = selection.getSelectedChild(0)
+                            orca.setLocusOfFocus(event, newFocus)
+                            self.pointOfReference['activeDescendantInfo'] = \
+                                [target, newFocus.getIndexInParent()]
+
+                        return
+
+        default.Script.onTextInserted(self, event)
+
 # Values used to construct a time string for calendar appointments.
 #
 timeIncrements = {}
