@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2008 Sun Microsystems Inc.
+# Copyright 2004-2009 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -21,10 +21,10 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 4433 2009-01-19 15:46:14Z wwalker $"
-__version__   = "$Revision: 4433 $"
-__date__      = "$Date: 2009-01-19 10:46:14 -0500 (Mon, 19 Jan 2009) $"
-__copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
+__copyright__ = "Copyright (c) 2004-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import os
@@ -82,7 +82,7 @@ import pyatspi
 import debug
 from acss import ACSS
 from orca_i18n import _           # for gettext support
-from orca_i18n import Q_          # to provide qualified translatable strings
+from orca_i18n import C_          # to provide qualified translatable strings
 
 # These are the settings that Orca supports the user customizing.
 #
@@ -240,6 +240,13 @@ BRAILLE_ROLENAME_STYLE_SHORT = 0 # three letter abbreviations
 BRAILLE_ROLENAME_STYLE_LONG  = 1 # full rolename
 brailleRolenameStyle    = BRAILLE_ROLENAME_STYLE_LONG
 
+# Roles to force to be displayed even when the verbosity level
+# is not verbose.
+#
+brailleForceRoles = [pyatspi.ROLE_COMBO_BOX,
+                     pyatspi.ROLE_MENU,
+                     pyatspi.ROLE_TEAROFF_MENU_ITEM]
+
 # Braille Selection Indicator (see brailleSelectorIndicator).
 # The values represent the character to be used in the attrOr
 # field of brlAPI's writeStruct.
@@ -303,7 +310,7 @@ if os.getenv("DBUS_SESSION_BUS_ADDRESS"):
 else:
     useDBus = False
 
-# Whether or not recording is enabled via the HTTP server. 
+# Whether or not recording is enabled via the HTTP server.
 #
 enableRemoteLogging     = False
 
@@ -341,7 +348,7 @@ enableSpeechIndentation = False
 
 # If True, enable braille.
 #
-enableBraille           = True
+enableBraille           = False
 
 # If True, add the hierarchical context of an object to the braille
 # line.  People with very large braille displays may want to set this
@@ -359,16 +366,6 @@ enableBrailleGrouping   = False
 #
 enableBrailleMonitor    = False
 
-# string to indicate end of printed line for braille displays:
-#
-disableBrailleEOL = False
-brailleEOLIndicator = " $l"
-
-# Strings used to indicate checkbox/radio button states in braille:
-#
-brailleCheckBoxIndicators = ["< >", "<x>", "<->"]
-brailleRadioButtonIndicators = ["& y", "&=y"]
-
 # If True, enable magnification.
 #
 enableMagnifier                  = False
@@ -609,7 +606,7 @@ keyboardLayout                  = GENERAL_KEYBOARD_LAYOUT_DESKTOP
 #
 outlineColor = [ 0xFFFF, 0x0000, 0x0000 ]
 
-# Thickness in pixels of the outline around the the current item in flat 
+# Thickness in pixels of the outline around the the current item in flat
 # review mode.
 #
 outlineThickness = 4
@@ -653,21 +650,6 @@ progressBarUpdateInterval = 10
 #
 presentReadOnlyText = True
 
-# Translators: this is used to indicate the user is in a text
-# area that is not editable.  It is meant to be spoken to the user.
-#
-# ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-#
-speechReadOnlyString = Q_("text|read only")
-
-# Translators: this is used to indicate the user is in a text
-# area that is not editable.  It is meant to be a short abbreviation
-# to be presented on the braille display.
-#
-# ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-#
-brailleReadOnlyString = Q_("text|rdonly")
-
 # The complete list of possible text attributes.
 #
 allTextAttributes = \
@@ -691,7 +673,8 @@ allTextAttributes = \
 
 enabledSpokenTextAttributes = \
     "size:; family-name:; weight:400; indent:0; underline:none; " \
-    "strikethrough:false; justification:left; style:normal;"
+    "strikethrough:false; justification:left; style:normal; " \
+    "paragraph-style:;"
 
 # The default set of text attributes to be brailled for the user. Specific
 # application scripts (or individual users can override these values if
@@ -724,6 +707,30 @@ textAttributesBrailleIndicator = TEXT_ATTR_BRAILLE_NONE
 #
 repeatCharacterLimit = 4
 
+# Tags associated with ARIA landmarks.
+#
+ariaLandmarks = [
+    "application",
+    "article",
+    "banner",
+    "complementary",
+    "contentinfo",
+    "definition",
+    "directory",
+    "document",
+    "grid",
+    "log",
+    "main",
+    "menubar",
+    "navigation",
+    "note",
+    "region",
+    "search",
+    "secondary",
+    "seealso",
+    "status",
+]
+
 # Script developer feature.  If False, just the default script
 # will be used.  Helps determine difference between custom
 # scripts and the default script behavior.
@@ -782,6 +789,11 @@ commFailureAttemptLimit = 5
 #
 gilSleepTime            = 0.00001
 
+# The value of the 'gil' parameter in the call to pyatspi.Registry.start
+# See http://bugzilla.gnome.org/show_bug.cgi?id=576954.
+#
+useGILIdleHandler       = False
+
 # If True, use the gidle __blockPreventor() code in atspi.py.
 #
 useBlockPreventor       = False
@@ -849,7 +861,7 @@ def setAccessibilityEnabled(enable):
     except:
         return False
 
-# Obtain/set information regarding whether Orca is autostarted for this 
+# Obtain/set information regarding whether Orca is autostarted for this
 # user at login time.
 #
 def isOrcaAutostarted():
@@ -917,7 +929,7 @@ settingsPackages          = ["app-settings"]
 # REMEMBER: to make something a package, the directory has to
 # have a __init__.py file in it.
 #
-scriptPackages          = ["orca-scripts", "scripts", 
+scriptPackages          = ["orca-scripts", "scripts",
                            "scripts.apps", "scripts.toolkits"]
 
 # A list that helps us map application names to script module
@@ -1066,7 +1078,7 @@ setScriptMapping(re.compile('gnome-help'), "yelp")
 #
 deprecatedMessages = False
 
-# Focus history length. We keep a reference to past focused accessibles to 
+# Focus history length. We keep a reference to past focused accessibles to
 # maximize on caching.
 #
 focusHistoryLength = 5
@@ -1080,22 +1092,22 @@ listenAllEvents = False
 #
 ignoredEventsList = ['object:bounds-changed']
 
-# Listen to Live Region events.  Tells Gecko.onChildrenChanged() and 
+# Listen to Live Region events.  Tells Gecko.onChildrenChanged() and
 # onTextInserted() event handlers to monitor these events for live region
-# changes.  
+# changes.
 #
 inferLiveRegions = True
 
 # Contracted braille support.
-# 
+#
 enableContractedBraille = False
 
 # Contracted braille table.
-# 
+#
 brailleContractionTable = ''
 
 # Use Collection Interface?
-# 
+#
 useCollection = True
 
 # Whether or not to speak the cell's coordinates when navigating
@@ -1130,13 +1142,13 @@ wrappedStructuralNavigation = True
 #
 enableMouseReview = False
 
-# Mouse dwell delay in milliseconds for mouse review mode. 
+# Mouse dwell delay in milliseconds for mouse review mode.
 # If the value is zero, the review will be read time.
 #
 mouseDwellDelay = 0
 
 # Maximum allowed drift while pointer is dwelling in mouse review mode.
-# 
+#
 mouseDwellMaxDrift = 3
 
 # The different modifiers/modifier masks associated with keybindings
@@ -1188,13 +1200,27 @@ defaultModifierMask = NON_LOCKING_MODIFIER_MASK
 #
 presentRequiredState = False
 
-# Translators: Certain objects (such as form controls on web pages)
-# can have STATE_REQUIRED set on them to inform the user that this
-# field must be filled out. This string is the default string which
-# will be spoken and displayed in braille to indicate this state is
-# present.
+########################################################################
+#                                                                      #
+# Sounds                                                               #
+#                                                                      #
+########################################################################
+
+# A dictionary where the keys are rolenames and the values are
+# filenames containing audio snippets.
 #
-brailleRequiredStateString = _("required")
+sounds = {}
+
+########################################################################
+#                                                                      #
+# Strings for speech and braille                                       #
+#                                                                      #
+########################################################################
+
+# Translators: "blank" is a short word to mean the
+# user has navigated to an empty line.
+#
+speechBlankString = _("blank")
 
 # Translators: Certain objects (such as form controls on web pages)
 # can have STATE_REQUIRED set on them to inform the user that this
@@ -1203,3 +1229,115 @@ brailleRequiredStateString = _("required")
 # present.
 #
 speechRequiredStateString = _("required")
+
+# Translators: this is used to indicate the user is in a text
+# area that is not editable.  It is meant to be spoken to the user.
+#
+speechReadOnlyString = C_("text", "read only")
+
+# Translators: this represents an item on the screen that has
+# been set insensitive (or grayed out).
+#
+speechInsensitiveString = _("grayed")
+
+# Translators: this represents the state of a checkbox.  It is meant
+# to be spoken to the user.
+#
+speechCheckboxIndicators = [_("not checked"),
+                            _("checked"),
+                            _("partially checked")]
+
+# Translators: this represents the state of a radio button.  It is
+# meant to be spoken to the user.
+#
+speechRadioButtonIndicators = [C_("radiobutton", "not selected"),
+                               C_("radiobutton", "selected")]
+
+# Translators: this represents the state of a toggle button.  It is
+# meant to be spoken to the user.
+#
+speechToggleButtonIndicators = [_("not pressed"), _("pressed")]
+
+# Translators: this represents the state of a node in a tree.
+# 'expanded' means the children are showing.  'collapsed' means the
+# children are not showing.
+#
+speechExpansionIndicators = [_("collapsed"), _("expanded")]
+
+# Translators: "multi-select" refers to a web form list
+# in which more than one item can be selected at a time.
+#
+speechMultiSelectString = _("multi-select")
+
+# Translators: this represents the depth of a node in a tree
+# view (i.e., how many ancestors a node has).  It is meant to be
+# spoken.
+#
+speechNodeLevelString = _("tree level %d")
+
+# Translators: this represents a list item in a document.
+# The nesting level is how 'deep' the item is (e.g., a
+# level of 2 represents a list item inside a list that's
+# inside another list). This is meant to be spoken.
+#
+speechNestingLevelString = _("Nesting level %d")
+
+# string to indicate end of printed line for braille displays:
+#
+disableBrailleEOL = False
+brailleEOLIndicator = " $l"
+
+# Translators: Certain objects (such as form controls on web pages)
+# can have STATE_REQUIRED set on them to inform the user that this
+# field must be filled out. This string is the default string which
+# will be spoken and displayed in braille to indicate this state is
+# present.
+#
+brailleRequiredStateString = _("required")
+
+# Translators: this is used to indicate the user is in a text
+# area that is not editable.  It is meant to be a short abbreviation
+# to be presented on the braille display.
+#
+brailleReadOnlyString = C_("text", "rdonly")
+
+# Translators: this represents an item on the screen that has
+# been set insensitive (or grayed out).
+#
+brailleInsensitiveString = _("grayed")
+
+# Strings used to indicate checkbox/radio button states in braille:
+#
+brailleCheckBoxIndicators = ["< >", "<x>", "<->"]
+brailleRadioButtonIndicators = ["& y", "&=y"]
+brailleToggleButtonIndicators = ["& y", "&=y"]
+
+# Translators: this represents the state of a node in a tree.
+# 'expanded' means the children are showing.  'collapsed' means the
+# children are not showing.
+#
+brailleExpansionIndicators = [_("collapsed"), _("expanded")]
+
+# Translators: this represents the depth of a node in a tree
+# view (i.e., how many ancestors a node has).  It is meant to
+# be presented on a braille display.
+#
+brailleNodeLevelString = _("TREE LEVEL %d")
+
+# Translators: this represents a list item in a document.
+# The nesting level is how 'deep' the item is (e.g., a
+# level of 2 represents a list item inside a list that's
+# inside another list).  It is meant to be presented on
+# the braille display.
+#
+brailleNestingLevelString = _("LEVEL %d")
+
+# String for delimiters between table cells
+#
+brailleTableCellDelimiter = " "
+
+# This is for bug #585417 - Allow pauses to be inserted into speech
+# output. We're keeping it separate for now until we get the pauses
+# sorted out just right.
+#
+useExperimentalSpeechProsody = True
